# -*- coding: utf-8 -*-
import logging
import os

def setup_logger():

    logger = logging.getLogger(__name__)
    logger.setLevel(logging.INFO)  # 设置日志级别为INFO

    if not logger.handlers:

        log_directory = os.path.join(os.path.dirname(__file__), 'logs')
        os.makedirs(log_directory, exist_ok=True)  # 确保日志目录存在
        file_handler = logging.FileHandler(os.path.join(log_directory, 'app.log'))
        file_handler.setLevel(logging.INFO)  # 文件记录INFO及以上级别的日志


        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(logging.INFO)  # 控制台输出INFO及以上级别的日志


        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
        file_handler.setFormatter(formatter)
        stream_handler.setFormatter(formatter)


        logger.addHandler(file_handler)
        logger.addHandler(stream_handler)

    return logger

def config_only_kzt_logger(logger_level="critical"):
    """
    初始化控制台输出logger，该方法不会写入本地日志文件
    :param logger_level: 设置日志级别，默认"critical"，其他级别："debug", "info", "warning", "error", "critical"
    :return: 日志句柄，用于打印日志
    """
    logger = logging.getLogger()
    if logger_level == "debug":
        logger.setLevel(logging.DEBUG)
    elif logger_level == "info":
        logger.setLevel(logging.INFO)
    elif logger_level == "warning":
        logger.setLevel(logging.WARNING)
    elif logger_level == "error":
        logger.setLevel(logging.ERROR)
    else:
        logger.setLevel(logging.CRITICAL)

    formatter = logging.Formatter(fmt='%(asctime)s  %(filename)s  [line:%(lineno)d]  %(levelname)s :%(message)s', datefmt='%Y-%m-%d %H:%M:%S')
    # 控制台的输出
    ch = logging.StreamHandler()
    ch.setFormatter(formatter)
    # 将输出的流 添加给logger
    logger.addHandler(ch)
    return logger


# -*- coding: utf-8 -*-
import threading
import time

from loongfei_utils.utils import StrUtils
from loongfei_utils.MultiThreading import ExecuteList
from copy import deepcopy
from Crypto.Cipher import AES

import shutil
import requests as rq
import re
import os


class TSCrawlingToMp4:
    """
    非加密ts文件抓取抓取
    """
    # 需要抓取的链接列表对应的名字列表，用于创建本地m3u8文件，并按次顺序合并TS文件
    ts_name_list = []
    # 需要抓取的链接列表
    crawling_url_list = []
    # 因个别网站文件名字超长，导致windows系统无法保存，因此采用映射关系保存
    name_and_name_rel = {}
    # 生成随机字符串使用
    default_ts_count = 100000
    # 下载错误的链接
    error_download_list = {}
    # 是否继续下载错误链接
    if_continue_download_error_url = True

    def __init__(self, url, path, zh_file_name, logger):
        """
        :param url: 视频对应的m3u8地址
        :param path: 要存放的路径，不能是中文
        :param zh_file_name: 将合并后采用英文名字的视频转移并命名为中文名字
        :param logger: 传入日志句柄，用于日志输出
        """
        self.error_retry_counts = 3
        self.count = 10
        self.url = url
        self.path = path
        self.zh_file_name = zh_file_name
        self.__init_file_path()
        self.logger = logger

    def __init_file_path(self):
        # 获得一个随机值，用来设置临时文件夹名称及临时mp4命名
        m3u8_str = StrUtils.init_random_tr("m3u8")
        # 存放的路径上创建新的文件夹，存放TS文件及m3u8文件，视频生成后将被删除
        if not self.path.endswith("/"):
            self.path = self.path + "/"
        self.save_ts_file_dir = self.path + m3u8_str + "/"
        # TS视频合并后的英文名字，用于ffmpeg处理，ffmpeg不支持中文路径
        self.en_file_name = m3u8_str
        # 判断存储TS的临时文件是否存在，不存在则创建
        if not os.path.exists(self.save_ts_file_dir):
            os.makedirs(self.save_ts_file_dir)
        # m3u8文件路径
        self.m3u8_file_path = self.save_ts_file_dir + self.en_file_name + ".txt"
        # 合并后的 mp4 英文路径
        self.en_mp4_view_path = self.save_ts_file_dir + self.en_file_name + ".mp4"
        # 合并后的 mp4 中文路径
        self.zh_mp4_view_path = self.path + self.zh_file_name + ".mp4"

    def __init_new_file_name(self):
        lock = threading.Lock()
        with lock:
            self.default_ts_count += 1
            new_ts_name = "ts_movie_" + str(self.default_ts_count) + ".ts"
        return new_ts_name

    def __init_crawling_ts_list(self):
        crawling_data = rq.get(self.url)
        self.data_body = crawling_data.text
        pattern = r"\S+\.ts"
        ts_item = ""
        ts_item_list = re.findall(pattern, self.data_body)
        if len(ts_item_list) > 0:
            ts_item = ts_item_list[0]
        if ts_item.startswith("http") or ts_item.startswith("HTTP"):
            self.crawling_url_list = deepcopy(ts_item_list)
            for url in ts_item_list:
                ts_name = url.split('/')[-1]
                new_ts_name = self.__init_new_file_name()
                self.name_and_name_rel[ts_name] = new_ts_name
                self.ts_name_list.append(new_ts_name)
        else:
            # self.ts_name_list = deepcopy(ts_item_list)
            for ts_name in ts_item_list:
                if not ts_name.startswith("/"):
                    crawling_url = re.sub("(?<=/)[^/]*.m3u8$", ts_name, self.url)
                    self.crawling_url_list.append(crawling_url)
                    new_ts_name = self.__init_new_file_name()
                    self.name_and_name_rel[ts_name] = new_ts_name
                    self.ts_name_list.append(new_ts_name)
        self.__init_decode_object()

    def __init_decode_object(self):
        # 获取加密算法类型
        encryption_key_str = ""
        encryption_key_str_list = re.findall("(?<=METHOD=)[^,]*(?=,)", self.data_body)
        if len(encryption_key_str_list) > 0:
            encryption_key_str = encryption_key_str_list[0]
        if encryption_key_str == "" or encryption_key_str is None:
            self.if_encryption = False
        else:
            self.if_encryption = True
            # 获得IV偏移量
            iv_temp_list = re.findall("(?<=IV=).*", self.data_body)
            if len(iv_temp_list) == 0:
                self.IV = ""
            else:
                self.IV = iv_temp_list[0].replace("0x", "")[:16].encode()
            # 获取TS解密key
            encryption_key_uri = ""
            encryption_key_uri_list = re.findall("(?<=URI=\")[^\"]*(?=\")", self.data_body)
            if len(encryption_key_uri_list) > 0:
                encryption_key_uri = encryption_key_uri_list[0]
            if encryption_key_uri.startswith("http") or encryption_key_uri.startswith("HTTP"):
                encryption_key_url = encryption_key_uri
            else:
                encryption_key_url = re.sub("(?<=/)[^/]*.m3u8$", encryption_key_uri, self.url)
            self.encryption_key = rq.get(encryption_key_url).content
            # 根据加密算法初始化解密对象
            if encryption_key_str == "AES-128":
                decode_obj = AES.new(self.encryption_key, AES.MODE_CBC, self.IV)
                self.decode_obj = decode_obj

    def __download_ts_video(self, crawling_url):
        if_success = True
        ts_name = crawling_url.split('/')[-1]
        new_ts_name = self.name_and_name_rel[ts_name]
        save_ts_path = self.save_ts_file_dir + new_ts_name
        try:
            with open(save_ts_path, 'wb+') as f:
                if self.if_encryption:
                    f.write(self.decode_obj.decrypt(rq.get(crawling_url, timeout=(60, 600)).content))
                else:
                    f.write(rq.get(crawling_url, timeout=(60, 600)).content)
        except BaseException as e:
            self.logger.error("---> 下载文件出错，错误信息为：{}".format(e))
            if_success = False
        return if_success

    def __create_ts_info_file(self):
        for error_download_item in self.error_download_list:
            ts_name = error_download_item.split('/')[-1]
            # self.logger.info("~~~~~ts_name: {}".format(ts_name))
            new_ts_name = self.name_and_name_rel[ts_name]
            # self.logger.info("~~~~~new_ts_name: {}".format(new_ts_name))
            # self.logger.info("~~~~~self.ts_name_list: {}".format(self.ts_name_list))
            self.ts_name_list.remove(new_ts_name)
        # self.logger.info("~~~after remove error items , self.ts_file_list:" + str(self.ts_name_list))
        with open(self.m3u8_file_path, 'w+') as f:
            for ts_name_temp in self.ts_name_list:
                f.write(f"file \'{ts_name_temp}\'" + "\n")

    def __hebing_ts(self):
        self.logger.info("---> m3u8文件地址：" + self.m3u8_file_path)
        self.logger.info("---> 英文视频地址：" + self.en_mp4_view_path)
        os.system("ffmpeg -f concat -i " + self.m3u8_file_path + " -c copy " + self.en_mp4_view_path)
        self.logger.info("---> 视频合并完成，开始转移视频，并重命名，新的文件地址为：" + self.zh_mp4_view_path)
        shutil.move(self.en_mp4_view_path, self.zh_mp4_view_path)
        self.logger.info("---> 转移完成，开始删除原始视频TS文件~~~")
        shutil.rmtree(self.save_ts_file_dir)

    def __retry_download_error_ts_video(self):
        time.sleep(3)
        if self.if_continue_download_error_url:
            if self.error_retry_counts <= 0 or len(self.error_download_list) <= 0:
                self.if_continue_download_error_url = False
        while self.if_continue_download_error_url:
            self.logger.info(f"---> 倒数第 {self.error_retry_counts} 次尝试,需重新下载 {len(self.error_download_list)} 个错误链接：" + str(self.error_download_list))
            # 初始化多线程类
            execute_list_temp = ExecuteList(self.__download_ts_video, self.error_download_list, self.count, self.logger)
            # 开启多线程
            execute_list_temp.start_thread()
            self.error_download_list.clear()
            self.error_download_list = deepcopy(set(execute_list_temp.getting_error_list()))
            self.error_retry_counts -= 1
            if self.error_retry_counts > 0 and len(self.error_download_list) > 0:
                self.__retry_download_error_ts_video()
            else:
                self.if_continue_download_error_url = False

    def do_crawling(self, count, error_retry_counts=3):
        """
        执行爬取
        :param count: 开启线程数
        :param error_retry_counts: 如果抓取错误，对于错误的请求，重新抓取次数，默认3次
        """
        # 基于m3u8地址获取TS对应的m3u8文件并整理成TS名字序列
        self.__init_crawling_ts_list()
        self.count = count
        self.error_retry_counts = error_retry_counts
        # 基于TS名字序列生产保存路径序列
        self.logger.info(self.ts_name_list)
        # self.logger.info(self.savePathList)
        self.logger.info(self.crawling_url_list)
        # 初始化多线程类
        execute_list = ExecuteList(self.__download_ts_video, self.crawling_url_list, self.count, self.logger)
        # 开启多线程
        execute_list.start_thread()
        if len(execute_list.getting_error_list()) > 0:
            self.error_download_list.clear()
            self.error_download_list = deepcopy(set(execute_list.getting_error_list()))
            self.__retry_download_error_ts_video()
        # 在本地生成txt，用来保存ts的合并顺序
        self.__create_ts_info_file()
        # 合并TS文件生成英文名称的mp4，转移正式mp4文件并以中文名命名同时删除ts临时文件
        self.__hebing_ts()
        self.logger.info(f"---> {self.zh_file_name} ，下载完成~~~~")



import re

from loongfei_utils.MultiThreading import ExecuteList
from loongfei_utils.utils import LoggerUtils
from loongfei_utils.utils import StrUtils
from loongfei_utils.M3u8Crawer import TSCrawlingToMp4

a = ['1s','2s','3','4','5','6','7','8']
logger = LoggerUtils.config_only_kzt_logger("info")
def do_a(item):
    logger.info(item)
    return True
# a.clear()
# aa = ExecuteList(do_a, a, 3, logger)
# aa.start_thread()
# print(str(aa.getting_error_list()))
#
# print(type(do_a))
# print(StrUtils.init_random_tr("sss"))

# cd = TSCrawlingToMp4(url="https://vip.lz-cdn.com/20220731/26529_5862e7cc/1000k/hls/mixed.m3u8",
#                              path="D:/Downloads/view/",
#                              zh_file_name="飞狐外传",
#                              logger=logger)
# cd.do_crawling(100,5)

# str = """
#       适合m3u8格式为：
# #EXTM3U
# #EXT-X-VERSION:3
# #EXT-X-TARGETDURATION:20
# #EXT-X-MEDIA-SEQUENCE:0
# #EXT-X-KEY:METHOD=AES-128,URI="enc.key",IV=0x00000000000000000000000000000000
# #EXTINF:20.333333,
# https://p.jisuts.com:999/hls/232/20240211/2326858/plist0.ts
# #EXTINF:1.125000,
#       """

# path='D:/Downloads/view/m3u8_20240623002855656747_8959/h5bo97v98l0rb8825afmfwzgg3sqwqgfzrim3wgnk7ex5pi20htskgvz3km7ji4zkkqfnzka0vsqu4e0lq2ari69qwkgpmfnh951txhkxkp6266ldt8pwbc1f1gsf5zrnezz0t4fql3y1kk5zw9q8zizsxfq0l7x8s5ea34khbgaf52j0an1ky7hi32alv98ijjdv7qnwttdbfx0yjpc6xcv6b6wiibrw7rcyqpeesch78gbr6ty0dptou0vzcvjup4faxa0d5736uh8lzfu2f20t08jh67itzz5imek9.ts'
# with open(path,'w+') as f:
#     f.write("1")

# path='D:/Downloads/view/m3u8_20240623002855656747_8959/h5bo97v98l0rb8825afmfwzgg3sqw0t4fql3yky7hi32apuh8lzfu2f20t08jh67itzz5imek9.ts'
# with open(path,'w+') as f:
#     f.write("1")

aaa = 5
aaa -= 1
print(aaa)




# -*- coding: utf-8 -*-
import threading
import time
from types import FunctionType


class ExecuteList:

    __function__ = None  # 处理序列数据的方法
    __need_execute_list__ = []  # 需要处理的数据序列
    __executing_list__ = []  # 正在处理中的数据序列
    __execute_error_list__ = []  # 处理错误的数据序列
    __thread_list__ = []  # 正在执行的线程列表
    __thread_num__ = 0  # 启用的线程数
    lock = threading.Lock()

    """
    fun：实际的处理过程，引入外部函数；需要返回处理结果:True or False
    need_execute_list：需要多线程处理的序列；
    thread_num：启用的线程数；
    logger：日志输出
    """

    def __init__(self, func:FunctionType, need_execute_list:list, thread_num:int, logger:object):
        """
        :param fun: 处理数据的函数
        :param need_execute_list: 需要被处理的数据
        :param thread_num:  启动的线程数
        :param logger:  打印日志logger
        """
        self.__function__ = func
        self.__need_execute_list__ = need_execute_list
        self.__thread_num__ = thread_num
        self.logger = logger

    def start_thread(self):
        if len(self.__need_execute_list__) < self.__thread_num__:
            self.__thread_num__ = len(self.__need_execute_list__)
        for i in range(self.__thread_num__):
            thread_name = "loongfei-thread-" + str(i)
            self.__thread_list__.append(thread_name)
            threading.Thread(target=self.__executing_list_item__, args=(thread_name,)).start()
        if_continue = True
        while if_continue:
            if len(self.__thread_list__) > 0:
                self.logger.info("---> 剩余线程：" + str(self.__thread_list__))
                time.sleep(10)
            else:
                if_continue = False
                self.logger.info("---> 多线程执行完毕，准备结束~~~")

    def __getting_executing_data__(self, execute_type, execute_list_item):
        """
        :param execute_type: 处理类型，1：取；2：销毁；3：添加至错误队列
        :param execute_list_item:
        :return:
        """
        with self.lock:
            if execute_type == "1":
                if len(self.__need_execute_list__) > 0:
                    execute_list_item = self.__need_execute_list__.pop()
                    self.__executing_list__.append(execute_list_item)
                else:
                    execute_list_item = ""
            elif execute_type == "2":
                self.__executing_list__.remove(execute_list_item)
            else:
                self.__executing_list__.remove(execute_list_item)
                self.__execute_error_list__.append(execute_list_item)
        return execute_list_item

    def __executing_list_item__(self, thread_name):
        continue_do = True
        while continue_do:
            execute_list_item = self.__getting_executing_data__("1", "")
            if execute_list_item != "":
                self.logger.info("---> 线程：“" + thread_name + "” ，获取数据 “" + str(execute_list_item) + "” 并开始处理~~~")
                execute_result = None
                try:
                    execute_result = self.__function__(execute_list_item)
                except BaseException as e:
                    self.__getting_executing_data__("3", execute_list_item)
                    self.logger.info("---> 线程：“" + thread_name + "” ，处理数据 “" + str(execute_list_item) + "” 失败，错误信息{0}~~~".format(e))
                else:
                    if not execute_result:
                        self.__getting_executing_data__("3", execute_list_item)
                        self.logger.error("---> 线程：“" + thread_name + "” ，处理数据 “" + str(execute_list_item) + "” 后，处理方法返回错误，请排查处理方法~~~")
                    else:
                        self.__getting_executing_data__("2", execute_list_item)
                        self.logger.info("---> 线程：“" + thread_name + "” ，处理数据 “" + str(execute_list_item) + "” 完毕~~~")

            else:
                continue_do = False
                self.logger.info("---> 线程：“" + thread_name + "” ，执行完毕，即将关闭~~~")
                self.__thread_list__.remove(thread_name)

    def getting_error_list(self):
        return self.__execute_error_list__



# -*- coding: utf-8 -*-
import datetime
import random
import threading


def init_random_tr(start_str="temp"):
    """
    初始化随机字符串，该方法为线程安全的
    :param start_str: 自定义开头字符串，如果空则默认为“temp”开头
    :return: 返回新的字符串，格式为：开头字符串_时间字符串_1000到9999的随机数
    """
    lock = threading.Lock()
    new_str = start_str
    with lock:
        time_str = datetime.datetime.now().strftime("%Y%m%d%H%M%S%f")
        random_str = str(random.randint(1000, 9999))
        new_str = new_str + "_" + time_str + "_" + random_str
    return new_str
